@@ -15,19 +15,19 @@ introspection providers.
 import functools
 import inspect
 import logging
-import os
-from pkg_resources import parse_version, iter_entry_points
+import sys
 from typing import List, Union
+import weakref
 
 # Third-party imports
-from qtpy.QtCore import QMutex, QMutexLocker, QTimer, Slot, Signal
+from packaging.version import parse
+from qtpy.QtCore import QRecursiveMutex, QMutexLocker, QTimer, Slot, Signal
 
 # Local imports
-from spyder.config.manager import CONF
 from spyder.api.plugins import SpyderPluginV2, Plugins
 from spyder.api.plugin_registration.decorators import (
     on_plugin_available, on_plugin_teardown)
-from spyder.config.base import _, running_under_pytest
+from spyder.api.translations import _
 from spyder.config.user import NoDefault
 from spyder.plugins.completion.api import (CompletionRequestTypes,
                                            SpyderCompletionProvider,
@@ -35,6 +35,13 @@ from spyder.plugins.completion.api import (CompletionRequestTypes,
 from spyder.plugins.completion.confpage import CompletionConfigPage
 from spyder.plugins.completion.container import CompletionContainer
 
+# See compatibility note on `group` keyword:
+# https://docs.python.org/3/library/importlib.metadata.html#entry-points
+if sys.version_info < (3, 10):  # pragma: no cover
+    from importlib_metadata import entry_points
+else:  # pragma: no cover
+    from importlib.metadata import entry_points
+
 
 logger = logging.getLogger(__name__)
 
@@ -62,15 +69,21 @@ class CompletionPlugin(SpyderPluginV2):
     set of :class:`SpyderCompletionProvider` instances that are discovered
     and registered via entrypoints.
 
-    This plugin can assume that `fallback`, `snippets`, `lsp` and `kite`
+    This plugin can assume that `fallback`, `snippets` and `lsp`
     completion providers are available, since they are included as part of
     Spyder.
     """
 
     NAME = 'completions'
     CONF_SECTION = 'completions'
-    REQUIRES = [Plugins.Preferences, Plugins.MainInterpreter]
-    OPTIONAL = [Plugins.Application, Plugins.StatusBar, Plugins.MainMenu]
+    REQUIRES = [Plugins.Preferences]
+    OPTIONAL = [
+        Plugins.MainInterpreter,
+        Plugins.MainMenu,
+        Plugins.IPythonConsole,
+        Plugins.PythonpathManager,
+        Plugins.StatusBar,
+    ]
 
     CONF_FILE = False
 
@@ -123,9 +136,15 @@ class CompletionPlugin(SpyderPluginV2):
         New PythonPath settings.
     """
 
-    sig_interpreter_changed = Signal()
+    _sig_interpreter_changed = Signal(str)
     """
-    This signal is used to report changes on the main Python interpreter.
+    This private signal is used to handle changes in the Python interpreter
+    done by other plugins.
+
+    Parameters
+    ----------
+    path: str
+        Path to the new interpreter.
     """
 
     sig_language_completions_available = Signal(dict, str)
@@ -153,20 +172,6 @@ class CompletionPlugin(SpyderPluginV2):
         Path to a file to open with the editor.
     """
 
-    sig_editor_rpc = Signal(str, tuple, dict)
-    """
-    This signal is used to perform remote calls in the editor.
-
-    Parameters
-    ----------
-    method: str
-        Name of the method to call in the editor.
-    args: tuple
-        Tuple containing the positional arguments to perform the call.
-    kwargs: dict
-        Dictionary containing the optional arguments to perform the call.
-    """
-
     sig_stop_completions = Signal(str)
     """
     This signal is used to stop completion services on other Spyder plugins
@@ -212,7 +217,7 @@ class CompletionPlugin(SpyderPluginV2):
         self.req_id = 0
 
         # Lock to prevent concurrent access to requests mapping
-        self.collection_mutex = QMutex(QMutex.Recursive)
+        self.collection_mutex = QRecursiveMutex()
 
         # Completion request priority
         self.source_priority = {}
@@ -232,16 +237,10 @@ class CompletionPlugin(SpyderPluginV2):
 
         # Find and instantiate all completion providers registered via
         # entrypoints
-        for entry_point in iter_entry_points(COMPLETION_ENTRYPOINT):
+        for entry_point in entry_points(group=COMPLETION_ENTRYPOINT):
             try:
-                # This absolutely ensures that the Kite provider won't be
-                # loaded. For instance, it can happen when you have an older
-                # Spyder version installed, but you're running it with
-                # bootstrap.
-                if 'kite' in entry_point.name:
-                    continue
                 logger.debug(f'Loading entry point: {entry_point}')
-                Provider = entry_point.resolve()
+                Provider = entry_point.load()
                 self._instantiate_and_register_provider(Provider)
             except Exception as e:
                 logger.warning('Failed to load completion provider from entry '
@@ -257,35 +256,22 @@ class CompletionPlugin(SpyderPluginV2):
             CompletionConfigPage, providers=conf_providers)
         self.ADDITIONAL_CONF_TABS = {'completions': conf_tabs}
 
-    # ---------------- Public Spyder API required methods ---------------------
+    # ---- SpyderPluginV2 API
     @staticmethod
     def get_name() -> str:
         return _('Completion and linting')
 
-    def get_description(self) -> str:
-        return _('This plugin is in charge of handling and dispatching, as '
-                 'well as of receiving the responses of completion and '
-                 'linting requests sent to multiple providers.')
+    @staticmethod
+    def get_description() -> str:
+        return _('Handle code completion, analysis, formatting, introspection, '
+                 'folding and more via the Language Server Protocol and other '
+                 'providers.')
 
-    def get_icon(self):
-        return self.create_icon('completions')
+    @classmethod
+    def get_icon(cls):
+        return cls.create_icon('completions')
 
     def on_initialize(self):
-        self.sig_interpreter_changed.connect(self.update_completion_status)
-
-        if self.main:
-            self.main.sig_pythonpath_changed.connect(
-                self.sig_pythonpath_changed)
-
-        # Do not start providers on tests unless necessary
-        if running_under_pytest():
-            if not os.environ.get('SPY_TEST_USE_INTROSPECTION'):
-                # Prevent providers from receiving configuration updates
-                for provider_name in self.providers:
-                    provider_info = self.providers[provider_name]
-                    CONF.unobserve_configuration(provider_info['instance'])
-                return
-
         self.start_all_providers()
 
     @on_plugin_available(plugin=Plugins.Preferences)
@@ -296,14 +282,13 @@ class CompletionPlugin(SpyderPluginV2):
     @on_plugin_available(plugin=Plugins.MainInterpreter)
     def on_maininterpreter_available(self):
         maininterpreter = self.get_plugin(Plugins.MainInterpreter)
-        mi_container = maininterpreter.get_container()
 
-        # connect signals
-        self.completion_status.sig_open_preferences_requested.connect(
-            mi_container.sig_open_preferences_requested)
-
-        mi_container.sig_interpreter_changed.connect(
-            self.sig_interpreter_changed)
+        # This will allow people to change the interpreter used for completions
+        # if they disable the IPython console.
+        if not self.is_plugin_enabled(Plugins.IPythonConsole):
+            maininterpreter.sig_interpreter_changed.connect(
+                self._sig_interpreter_changed
+            )
 
     @on_plugin_available(plugin=Plugins.StatusBar)
     def on_statusbar_available(self):
@@ -311,13 +296,6 @@ class CompletionPlugin(SpyderPluginV2):
         self.statusbar = self.get_plugin(Plugins.StatusBar)
         for sb in container.all_statusbar_widgets():
             self.statusbar.add_status_widget(sb)
-        self.statusbar.add_status_widget(self.completion_status)
-
-    @on_plugin_available(plugin=Plugins.Application)
-    def on_application_available(self):
-        application = self.get_plugin(Plugins.Application)
-        self.sig_restart_requested.connect(
-            application.sig_restart_requested)
 
     @on_plugin_available(plugin=Plugins.MainMenu)
     def on_mainmenu_available(self):
@@ -331,6 +309,19 @@ class CompletionPlugin(SpyderPluginV2):
         for args, kwargs in self.items_to_add_to_application_menus:
             main_menu.add_item_to_application_menu(*args, **kwargs)
 
+    @on_plugin_available(plugin=Plugins.PythonpathManager)
+    def on_pythonpath_manager_available(self):
+        pythonpath_manager = self.get_plugin(Plugins.PythonpathManager)
+        pythonpath_manager.sig_pythonpath_changed.connect(
+            self.sig_pythonpath_changed)
+
+    @on_plugin_available(plugin=Plugins.IPythonConsole)
+    def on_ipython_console_available(self):
+        ipyconsole = self.get_plugin(Plugins.IPythonConsole)
+        ipyconsole.sig_interpreter_changed.connect(
+            self._sig_interpreter_changed
+        )
+
     @on_plugin_teardown(plugin=Plugins.Preferences)
     def on_preferences_teardown(self):
         preferences = self.get_plugin(Plugins.Preferences)
@@ -339,10 +330,12 @@ class CompletionPlugin(SpyderPluginV2):
     @on_plugin_teardown(plugin=Plugins.MainInterpreter)
     def on_maininterpreter_teardown(self):
         maininterpreter = self.get_plugin(Plugins.MainInterpreter)
-        mi_container = maininterpreter.get_container()
 
-        mi_container.sig_interpreter_changed.disconnect(
-            self.sig_interpreter_changed)
+        # We only connect to this signal if the IPython console is not enabled
+        if not self.is_plugin_enabled(Plugins.IPythonConsole):
+            maininterpreter.sig_interpreter_changed.disconnect(
+                self._sig_interpreter_changed
+            )
 
     @on_plugin_teardown(plugin=Plugins.StatusBar)
     def on_statusbar_teardown(self):
@@ -375,6 +368,20 @@ class CompletionPlugin(SpyderPluginV2):
                 main_menu.remove_item_from_application_menu(
                     item_id, menu_id=menu_id)
 
+    @on_plugin_teardown(plugin=Plugins.PythonpathManager)
+    def on_pythonpath_manager_teardown(self):
+        pythonpath_manager = self.get_plugin(Plugins.PythonpathManager)
+        pythonpath_manager.sig_pythonpath_changed.disconnect(
+            self.sig_pythonpath_changed)
+
+    @on_plugin_teardown(plugin=Plugins.IPythonConsole)
+    def on_ipython_console_teardowm(self):
+        ipyconsole = self.get_plugin(Plugins.IPythonConsole)
+        ipyconsole.sig_interpreter_changed.disconnect(
+            self._sig_interpreter_changed
+        )
+
+    # ---- Public API
     def stop_all_providers(self):
         """Stop all running completion providers."""
         for provider_name in self.providers:
@@ -385,26 +392,29 @@ class CompletionPlugin(SpyderPluginV2):
 
     def can_close(self) -> bool:
         """Check if any provider has any pending task."""
-        can_close = False
         for provider_name in self.providers:
             provider_info = self.providers[provider_name]
             if provider_info['status'] == self.RUNNING:
                 provider = provider_info['instance']
-                provider_can_close = provider.can_close()
-                can_close |= provider_can_close
-        return can_close
+                if not provider.can_close():
+                    return False
+        return True
 
     def on_close(self, cancelable=False) -> bool:
         """Check if any provider has any pending task before closing."""
-        can_close = False
+        can_close = True
         for provider_name in self.providers:
             provider_info = self.providers[provider_name]
             if provider_info['status'] == self.RUNNING:
                 provider = provider_info['instance']
                 provider_can_close = provider.can_close()
-                can_close |= provider_can_close
+                can_close = can_close and provider_can_close
+                logger.debug(
+                    f"Provider {provider_name} can close: {provider_can_close}"
+                )
                 if provider_can_close:
                     provider.shutdown()
+
         return can_close
 
     def after_configuration_update(self, options: List[Union[tuple, str]]):
@@ -433,11 +443,6 @@ class CompletionPlugin(SpyderPluginV2):
                 elif option_name == 'provider_configuration':
                     providers_to_update |= {provider_name}
 
-        # Update entries in the source menu
-        # FIXME: Delete this after CONF is moved to an observer pattern.
-        # and the editor migration starts
-        self.sig_editor_rpc.emit('update_source_menu', (options,), {})
-
     def on_mainwindow_visible(self):
         for provider_name in self.providers:
             provider_info = self.providers[provider_name]
@@ -507,34 +512,6 @@ class CompletionPlugin(SpyderPluginV2):
                 self.get_container().remove_statusbar_widget(id_)
                 self.statusbar.remove_status_widget(id_)
 
-    @property
-    def completion_status(self):
-        return self.get_container().completion_status
-
-    @Slot()
-    def update_completion_status(self):
-        maininterpreter = self.get_plugin(Plugins.MainInterpreter)
-        mi_status = maininterpreter.get_container().interpreter_status
-
-        value = mi_status.value
-        tool_tip = mi_status._interpreter
-
-        if '(' in value:
-            value, _ = value.split('(')
-
-        if ':' in value:
-            kind, name = value.split(':')
-        else:
-            kind, name = value, ''
-        kind = kind.strip()
-        name = name.strip()
-
-        new_value = f'Completions: {kind}'
-        if name:
-            new_value += f'({name})'
-
-        self.completion_status.update_status(new_value, tool_tip)
-
     # -------- Completion provider initialization redefinition wrappers -------
     def gather_providers_and_configtabs(self):
         """
@@ -735,8 +712,8 @@ class CompletionPlugin(SpyderPluginV2):
 
         # Check if there were any version changes between configurations
         provider_config = provider_configurations[provider_name]
-        provider_conf_version = parse_version(Provider.CONF_VERSION)
-        current_conf_version = parse_version(provider_config['version'])
+        provider_conf_version = parse(Provider.CONF_VERSION)
+        current_conf_version = parse(provider_config['version'])
 
         current_conf_values = provider_config['values']
         current_defaults = provider_config['defaults']
@@ -828,8 +805,9 @@ class CompletionPlugin(SpyderPluginV2):
 
         self.sig_pythonpath_changed.connect(
             provider_instance.python_path_update)
-        self.sig_interpreter_changed.connect(
-            provider_instance.main_interpreter_changed)
+        self._sig_interpreter_changed.connect(
+            provider_instance.interpreter_changed
+        )
 
     def _instantiate_and_register_provider(
             self, Provider: SpyderCompletionProvider):
@@ -943,7 +921,15 @@ class CompletionPlugin(SpyderPluginV2):
         """Start a given provider."""
         provider_info = self.providers[provider_name]
         if provider_info['status'] == self.STOPPED:
-            provider_info['instance'].start()
+            provider_instance = provider_info['instance']
+            provider_instance.start()
+            for language in self.language_status:
+                language_providers = self.language_status[language]
+                language_providers[provider_name] = (
+                    provider_instance.start_completion_services_for_language(
+                        language
+                    )
+                )
 
     def shutdown_provider_instance(self, provider_name: str):
         """Shutdown a given provider."""
@@ -951,6 +937,10 @@ class CompletionPlugin(SpyderPluginV2):
         if provider_info['status'] == self.RUNNING:
             provider_info['instance'].shutdown()
             provider_info['status'] = self.STOPPED
+            for language in self.language_status:
+                language_providers = self.language_status[language]
+                if provider_name in language_providers:
+                    language_providers[provider_name] = False
 
     # ---------- Methods to create/access graphical elements -----------
     def create_action(self, *args, **kwargs):
@@ -1020,7 +1010,7 @@ class CompletionPlugin(SpyderPluginV2):
         self.requests[req_id] = {
             'language': language,
             'req_type': req_type,
-            'response_instance': req['response_instance'],
+            'response_instance': weakref.ref(req['response_instance']),
             'sources': {},
             'timed_out': False,
         }
@@ -1224,7 +1214,7 @@ class CompletionPlugin(SpyderPluginV2):
         else:
             # Any empty response will be discarded and the completion
             # loop will wait for the next non-empty response.
-            # This should fix the scenario where Kite does not have a
+            # This should fix the scenario where a provider does not have a
             # response for a non-aggregated request but the LSP does.
             any_nonempty = any(request_responses['sources'].get(source)
                                for source in sorted_providers)
@@ -1239,7 +1229,7 @@ class CompletionPlugin(SpyderPluginV2):
         """
         request_responses = self.requests[req_id]
         req_type = request_responses['req_type']
-        response_instance = id(request_responses['response_instance'])
+        response_instance = id(request_responses['response_instance']())
         do_send = True
 
         # This is necessary to prevent sending completions for old requests
@@ -1248,7 +1238,7 @@ class CompletionPlugin(SpyderPluginV2):
             max_req_id = max(
                 [key for key, item in self.requests.items()
                  if item['req_type'] == req_type
-                 and id(item['response_instance']) == response_instance]
+                 and id(item['response_instance']()) == response_instance]
                 or [-1])
             do_send = (req_id == max_req_id)
 
@@ -1266,7 +1256,7 @@ class CompletionPlugin(SpyderPluginV2):
         """
         req_type = request_responses['req_type']
         req_id_responses = request_responses['sources']
-        response_instance = request_responses['response_instance']
+        response_instance = request_responses['response_instance']()
         logger.debug('Gather responses for {0}'.format(req_type))
 
         if req_type == CompletionRequestTypes.DOCUMENT_COMPLETION:
@@ -1275,7 +1265,8 @@ class CompletionPlugin(SpyderPluginV2):
             responses = self.gather_responses(req_type, req_id_responses)
 
         try:
-            response_instance.handle_response(req_type, responses)
+            if response_instance:
+                response_instance.handle_response(req_type, responses)
         except RuntimeError:
             # This is triggered when a codeeditor instance has been
             # removed before the response can be processed.
