@@ -16,14 +16,27 @@ from spyder.utils.environ import get_user_env
 
 
 def check_path(path):
-    """Check that `path` is not a [site|dist]-packages folder."""
+    """
+    Check that `path` is not a [site|dist]-packages folder or a Conda
+    distribution `pkgs` directory.
+    """
+    pattern_string = r'(ana|mini|micro)(conda|mamba|forge)\d*(/base)*/pkgs'
     if os.name == 'nt':
-        pat = re.compile(r'.*lib/(?:site|dist)-packages.*')
+        pattern_string = (
+            f'.*({pattern_string}'
+            r'|(l|L)ib/(site|dist)-packages).*'
+        )
     else:
-        pat = re.compile(r'.*lib/python.../(?:site|dist)-packages.*')
+        pattern_string = (
+            f'.*({pattern_string}'
+            r'|(lib|lib64)/'
+            r'(python|python\d+|python\d+\.\d+)/'
+            r'(site|dist)-packages).*'
+        )
+    pattern = re.compile(pattern_string, re.IGNORECASE)
 
     path_norm = path.replace('\\', '/')
-    return pat.match(path_norm) is None
+    return pattern.match(path_norm) is None
 
 
 def get_system_pythonpath():
